@@ -1,19 +1,18 @@
 /*
- * This file is part of MPlayer.
+ * This file is part of mpv.
  *
- * MPlayer is free software; you can redistribute it and/or modify
+ * mpv is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
- * MPlayer is distributed in the hope that it will be useful,
+ * mpv is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License along
- * with MPlayer; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+ * with mpv.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef MPLAYER_AF_H
@@ -62,24 +61,35 @@ struct af_instance {
     struct replaygain_data *replaygain_data;
     int (*control)(struct af_instance *af, int cmd, void *arg);
     void (*uninit)(struct af_instance *af);
-    /* flags is a bit mask of AF_FILTER_FLAG_* values
-     * returns 0 on success, negative value on error
+    /* Feed a frame. The frame is NULL if EOF was reached, and the filter
+     * should drain all remaining buffered data.
+     * Use af_add_output_frame() to output data. The optional filter_out
+     * callback can be set to produce output frames gradually.
      */
-    int (*filter)(struct af_instance *af, struct mp_audio *data, int flags);
+    int (*filter_frame)(struct af_instance *af, struct mp_audio *frame);
+    int (*filter_out)(struct af_instance *af);
     void *priv;
     struct mp_audio *data; // configuration and buffer for outgoing data stream
+
     struct af_instance *next;
     struct af_instance *prev;
     double delay; /* Delay caused by the filter, in seconds of audio consumed
                    * without corresponding output */
-    double mul; /* length multiplier: how much does this instance change
-                 * the number of samples passed though. (Ratio of input
-                 * and output, e.g. mul=4 => 1 sample becomes 4 samples) .*/
     bool auto_inserted; // inserted by af.c, such as conversion filters
+    char *label;
+
+    struct mp_audio fmt_in, fmt_out;
+
+    struct mp_audio **out_queued;
+    int num_out_queued;
+
+    struct mp_audio_pool *out_pool;
 };
 
 // Current audio stream
 struct af_stream {
+    int initialized; // 0: no, 1: yes, -1: attempted to, but failed
+
     // The first and last filter in the list
     struct af_instance *first;
     struct af_instance *last;
@@ -117,6 +127,7 @@ enum af_control {
     AF_CONTROL_SET_PAN_BALANCE,
     AF_CONTROL_GET_PAN_BALANCE,
     AF_CONTROL_SET_PLAYBACK_SPEED,
+    AF_CONTROL_SET_PLAYBACK_SPEED_RESAMPLE,
 };
 
 // Argument for AF_CONTROL_SET_PAN_LEVEL
@@ -129,20 +140,26 @@ struct af_stream *af_new(struct mpv_global *global);
 void af_destroy(struct af_stream *s);
 int af_init(struct af_stream *s);
 void af_uninit(struct af_stream *s);
-struct af_instance *af_add(struct af_stream *s, char *name, char **args);
-int af_filter(struct af_stream *s, struct mp_audio *data, int flags);
+struct af_instance *af_add(struct af_stream *s, char *name, char *label,
+                           char **args);
+int af_remove_by_label(struct af_stream *s, char *label);
+struct af_instance *af_find_by_label(struct af_stream *s, char *label);
 struct af_instance *af_control_any_rev(struct af_stream *s, int cmd, void *arg);
 void af_control_all(struct af_stream *s, int cmd, void *arg);
+void af_seek_reset(struct af_stream *s);
+
+void af_add_output_frame(struct af_instance *af, struct mp_audio *frame);
+int af_filter_frame(struct af_stream *s, struct mp_audio *frame);
+int af_output_frame(struct af_stream *s, bool eof);
+struct mp_audio *af_read_output_frame(struct af_stream *s);
+int af_make_writeable(struct af_instance *af, struct mp_audio *frame);
 
-double af_calc_filter_multiplier(struct af_stream *s);
 double af_calc_delay(struct af_stream *s);
 
 int af_test_output(struct af_instance *af, struct mp_audio *out);
 
 int af_from_dB(int n, float *in, float *out, float k, float mi, float ma);
-int af_to_dB(int n, float *in, float *out, float k);
 int af_from_ms(int n, float *in, int *out, int rate, float mi, float ma);
-int af_to_ms(int n, int *in, float *out, int rate);
 float af_softclip(float a);
 
 #endif /* MPLAYER_AF_H */
