@@ -8,21 +8,20 @@
  *
  * copyright (c) 2005 Alex Beregszaszi
  *
- * This file is part of MPlayer.
+ * This file is part of mpv.
  *
- * MPlayer is free software; you can redistribute it and/or modify
+ * mpv is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
- * MPlayer is distributed in the hope that it will be useful,
+ * mpv is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License along
- * with MPlayer; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+ * with mpv.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <stdio.h>
@@ -35,7 +34,7 @@
 // Data for specific instances of this filter
 typedef struct af_center_s
 {
-  int ch;		// Channel number which to insert the filtered data
+  int ch;               // Channel number which to insert the filtered data
 }af_center_t;
 
 // Initialization and runtime control
@@ -58,15 +57,20 @@ static int control(struct af_instance* af, int cmd, void* arg)
   return AF_UNKNOWN;
 }
 
-// Filter data through filter
-static int filter(struct af_instance* af, struct mp_audio* data, int flags)
+static int filter_frame(struct af_instance* af, struct mp_audio* data)
 {
-  struct mp_audio*    c   = data;	 // Current working data
+  if (!data)
+    return 0;
+  if (af_make_writeable(af, data) < 0) {
+    talloc_free(data);
+    return -1;
+  }
+  struct mp_audio*    c   = data;        // Current working data
   af_center_t*  s   = af->priv; // Setup for this instance
-  float*   	a   = c->planes[0];	 // Audio data
-  int		nch = c->nch;	 // Number of channels
-  int		len = c->samples*c->nch;	 // Number of samples in current audio block
-  int		ch  = s->ch;	 // Channel in which to insert the center audio
+  float*        a   = c->planes[0];      // Audio data
+  int           nch = c->nch;    // Number of channels
+  int           len = c->samples*c->nch;         // Number of samples in current audio block
+  int           ch  = s->ch;     // Channel in which to insert the center audio
   register int  i;
 
   // Run filter
@@ -75,18 +79,19 @@ static int filter(struct af_instance* af, struct mp_audio* data, int flags)
     a[i+ch] = (a[i]/2) + (a[i+1]/2);
   }
 
+  af_add_output_frame(af, data);
   return 0;
 }
 
 // Allocate memory and set function pointers
 static int af_open(struct af_instance* af){
   af->control=control;
-  af->filter=filter;
+  af->filter_frame = filter_frame;
   return AF_OK;
 }
 
 #define OPT_BASE_STRUCT af_center_t
-struct af_info af_info_center = {
+const struct af_info af_info_center = {
     .info = "Audio filter for adding a center channel",
     .name = "center",
     .flags = AF_FLAGS_NOT_REENTRANT,
