@@ -252,6 +252,7 @@ static const int alsa_to_mp_channels[][2] = {
     {SND_CHMAP_TRR,     MP_SP(TBR)},
     {SND_CHMAP_TRC,     MP_SP(TBC)},
     {SND_CHMAP_MONO,    MP_SP(FC)},
+    {SND_CHMAP_NA,      MP_SPEAKER_ID_NA},
     {SND_CHMAP_LAST,    MP_SPEAKER_ID_COUNT}
 };
 
@@ -275,6 +276,20 @@ static int find_alsa_channel(int mp_channel)
     return SND_CHMAP_UNKNOWN;
 }
 
+static int mp_chmap_from_alsa(struct mp_chmap *dst, snd_pcm_chmap_t *src)
+{
+    *dst = (struct mp_chmap) {0};
+
+    if (src->channels > MP_NUM_CHANNELS)
+        return -1;
+
+    dst->num = src->channels;
+    for (int c = 0; c < dst->num; c++)
+        dst->speaker[c] = find_mp_channel(src->pos[c]);
+
+    return 0;
+}
+
 static bool query_chmaps(struct ao *ao, struct mp_chmap *chmap)
 {
     struct priv *p = ao->priv;
@@ -285,14 +300,8 @@ static bool query_chmaps(struct ao *ao, struct mp_chmap *chmap)
         return false;
 
     for (int i = 0; maps[i] != NULL; i++) {
-        if (maps[i]->map.channels > MP_NUM_CHANNELS) {
-            MP_VERBOSE(ao, "skipping ALSA channel map with too many channels.\n");
-            continue;
-        }
-
-        struct mp_chmap entry = {.num = maps[i]->map.channels};
-        for (int c = 0; c < entry.num; c++)
-            entry.speaker[c] = find_mp_channel(maps[i]->map.pos[c]);
+        struct mp_chmap entry;
+        mp_chmap_from_alsa(&entry, &maps[i]->map);
 
         if (mp_chmap_is_valid(&entry)) {
             MP_VERBOSE(ao, "Got supported channel map: %s (type %s)\n",
@@ -404,7 +413,7 @@ alsa_error: ;
 #define INIT_OK 0
 #define INIT_ERROR -1
 #define INIT_BRAINDEATH -2
-static int init_device(struct ao *ao)
+static int init_device(struct ao *ao, bool second_try)
 {
     struct priv *p = ao->priv;
     int err;
@@ -492,11 +501,12 @@ static int init_device(struct ao *ao)
     }
 
     if (num_channels != ao->channels.num) {
+        int req = ao->channels.num;
         mp_chmap_from_channels_alsa(&ao->channels, num_channels);
         if (!mp_chmap_is_valid(&ao->channels))
             mp_chmap_from_channels(&ao->channels, 2);
-        MP_ERR(ao, "Couldn't get requested number of channels, fallback to %s.\n",
-               mp_chmap_to_str(&ao->channels));
+        MP_ERR(ao, "Couldn't get requested number of channels (%d), fallback "
+               "to %s.\n", req, mp_chmap_to_str(&ao->channels));
     }
 
     // Some ALSA drivers have broken delay reporting, so disable the ALSA
@@ -564,9 +574,8 @@ static int init_device(struct ao *ao)
         if (snd_pcm_chmap_print(alsa_chmap, sizeof(tmp), tmp) > 0)
             MP_VERBOSE(ao, "channel map reported by ALSA: %s\n", tmp);
 
-        struct mp_chmap chmap = {.num = alsa_chmap->channels};
-        for (int c = 0; c < chmap.num; c++)
-            chmap.speaker[c] = find_mp_channel(alsa_chmap->pos[c]);
+        struct mp_chmap chmap;
+        mp_chmap_from_alsa(&chmap, alsa_chmap);
 
         MP_VERBOSE(ao, "which we understand as: %s\n", mp_chmap_to_str(&chmap));
 
@@ -575,24 +584,14 @@ static int init_device(struct ao *ao)
         } else if (AF_FORMAT_IS_IEC61937(ao->format)) {
             MP_VERBOSE(ao, "using spdif passthrough; ignoring the channel map.\n");
         } else if (mp_chmap_is_valid(&chmap)) {
-            if (mp_chmap_equals(&chmap, &ao->channels)) {
-                MP_VERBOSE(ao, "which is what we requested.\n");
-            } else if (chmap.num == ao->channels.num) {
-                MP_VERBOSE(ao, "using the ALSA channel map.\n");
-                ao->channels = chmap;
-            } else {
-                MP_WARN(ao, "ALSA channel map conflicts with channel count!\n");
-            }
-        } else {
             // Is it one that contains NA channels?
-            struct mp_chmap chmap2 = {0};
-            for (int c = 0; c < alsa_chmap->channels; c++) {
-                int alsa_ch = alsa_chmap->pos[c];
-                if (alsa_ch != SND_CHMAP_NA)
-                    chmap2.speaker[chmap2.num++] = find_mp_channel(alsa_ch);
-            }
+            struct mp_chmap without_na = chmap;
+            mp_chmap_remove_na(&without_na);
 
-            if (mp_chmap_is_valid(&chmap2)) {
+            if (mp_chmap_is_valid(&without_na) &&
+                !mp_chmap_equals(&without_na, &chmap) &&
+                !second_try)
+            {
                 // Sometimes, ALSA will advertise certain chmaps, but it's not
                 // possible to set them. This can happen with dmix: as of
                 // alsa 1.0.28, dmix can do stereo only, but advertises the
@@ -609,10 +608,19 @@ static int init_device(struct ao *ao)
                 err = snd_pcm_close(p->alsa);
                 p->alsa = NULL;
                 CHECK_ALSA_ERROR("pcm close error");
-                ao->channels = chmap2;
+                ao->channels = without_na;
                 return INIT_BRAINDEATH;
             }
 
+            if (mp_chmap_equals(&chmap, &ao->channels)) {
+                MP_VERBOSE(ao, "which is what we requested.\n");
+            } else if (chmap.num == ao->channels.num) {
+                MP_VERBOSE(ao, "using the ALSA channel map.\n");
+                ao->channels = chmap;
+            } else {
+                MP_WARN(ao, "ALSA channel map conflicts with channel count!\n");
+            }
+        } else {
             MP_WARN(ao, "Got unknown channel map from ALSA.\n");
         }
 
@@ -683,9 +691,9 @@ static int init(struct ao *ao)
 
     MP_VERBOSE(ao, "using ALSA version: %s\n", snd_asoundlib_version());
 
-    int r = init_device(ao);
+    int r = init_device(ao, false);
     if (r == INIT_BRAINDEATH)
-        r = init_device(ao); // retry with normalized channel layout
+        r = init_device(ao, true); // retry with normalized channel layout
     return r == INIT_OK ? 0 : -1;
 }
 
