@@ -20,6 +20,7 @@
 #ifndef MP_AO_WASAPI_H_
 #define MP_AO_WASAPI_H_
 
+#include <stdbool.h>
 #include <audioclient.h>
 #include <audiopolicy.h>
 #include <mmdeviceapi.h>
@@ -30,10 +31,11 @@
 typedef struct change_notify {
     IMMNotificationClient client; /* this must be first in the structure! */
     LPWSTR monitored; /* Monitored device */
+    bool is_hotplug;
     struct ao *ao;
 } change_notify;
 
-HRESULT wasapi_change_init(struct ao* ao);
+HRESULT wasapi_change_init(struct ao* ao, bool is_hotplug);
 void wasapi_change_uninit(struct ao* ao);
 
 #define EXIT_ON_ERROR(hres)  \
@@ -41,28 +43,27 @@ void wasapi_change_uninit(struct ao* ao);
 #define SAFE_RELEASE(unk, release) \
               do { if ((unk) != NULL) { release; (unk) = NULL; } } while(0)
 
+enum wasapi_thread_state {
+    WASAPI_THREAD_FEED = 0,
+    WASAPI_THREAD_RESUME,
+    WASAPI_THREAD_RESET,
+    WASAPI_THREAD_SHUTDOWN
+};
+
 typedef struct wasapi_state {
     struct mp_log *log;
-    HANDLE threadLoop;
-
     /* Init phase */
     HRESULT init_ret;
-    HANDLE init_done;
+    HANDLE hInitDone;
     int share_mode;
 
-    HANDLE hUninit;
-
     /* volume control */
-    DWORD vol_hw_support, status;
+    DWORD vol_hw_support;
     float audio_volume;
     float previous_volume;
     float initial_volume;
 
-    /* Buffers */
-    size_t buffer_block_size; /* Size of each block in bytes */
-    UINT32 bufferFrameCount; /* wasapi buffer block size, number of frames, frame size at format.nBlockAlign */
-
-    /* WASAPI handles, owned by other thread */
+    /* WASAPI handles, owned by audio thread */
     IMMDevice *pDevice;
     IAudioClient *pAudioClient;
     IAudioRenderClient *pRenderClient;
@@ -71,15 +72,17 @@ typedef struct wasapi_state {
     IAudioSessionControl *pSessionControl;
     IMMDeviceEnumerator *pEnumerator;
 
-    HANDLE hFeed; /* wasapi event */
-    HANDLE hResume; /* signal audio thread to resume the stream */
-    HANDLE hReset; /* signal audio thread to reset the stream */
+    /* thread handles */
+    HANDLE hAudioThread; /* the thread itself */
+    HANDLE hWake; /* thread wakeup event */
+    atomic_int thread_state; /* enum wasapi_thread_state */
+
+    /* for setting the audio thread priority */
     HANDLE hTask; /* AV thread */
     DWORD taskIndex; /* AV task ID */
-    WAVEFORMATEXTENSIBLE format;
 
     /* WASAPI proxy handles, for Single-Threaded Apartment communication.
-       One is needed for each object that's accessed by a different thread. */
+       One is needed for each audio thread object that's accessed from the main thread. */
     ISimpleAudioVolume *pAudioVolumeProxy;
     IAudioEndpointVolume *pEndpointVolumeProxy;
     IAudioSessionControl *pSessionControlProxy;
@@ -97,10 +100,16 @@ typedef struct wasapi_state {
     atomic_ullong sample_count; /* the amount of samples per channel written to a GetBuffer buffer */
     LARGE_INTEGER qpc_frequency; /* frequency of windows' high resolution timer */
 
+    /* ao options */
     int opt_exclusive;
     int opt_list;
     char *opt_device;
 
+    /* format info */
+    WAVEFORMATEXTENSIBLE format;
+    size_t buffer_block_size; /* Size of each block in bytes */
+    UINT32 bufferFrameCount; /* wasapi buffer block size, number of frames, frame size at format.nBlockAlign */
+
     /* Don't use these functions directly in case
        they are unimplemented for some reason.
        (XP shouldn't be an issue since it doesn't support wasapi, maybe wine?)
