@@ -17,7 +17,7 @@
 
 #include <assert.h>
 #include "talloc.h"
-#include "bstr/bstr.h"
+#include "misc/bstr.h"
 #include "common/msg.h"
 #include "codecs.h"
 
