@@ -1,21 +1,20 @@
 /*
- * This file is part of MPlayer.
- *
  * Original author: Uoti Urpala
  *
- * MPlayer is free software; you can redistribute it and/or modify
+ * This file is part of mpv.
+ *
+ * mpv is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
- * MPlayer is distributed in the hope that it will be useful,
+ * mpv is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License along
- * with MPlayer; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+ * with mpv.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <stdlib.h>
@@ -134,22 +133,6 @@ error:
     return NULL;
 }
 
-static struct demuxer *open_file(char *filename, struct timeline *tl)
-{
-    struct MPOpts *opts = tl->global->opts;
-    struct demuxer *d = NULL;
-    struct stream *s = stream_open(filename, tl->global);
-    if (s) {
-        stream_enable_cache(&s, &opts->stream_cache);
-        d = demux_open(s, NULL, NULL, tl->global);
-    }
-    if (!d) {
-        MP_ERR(tl, "EDL: Could not open source file '%s'.\n", filename);
-        free_stream(s);
-    }
-    return d;
-}
-
 static struct demuxer *open_source(struct timeline *tl, char *filename)
 {
     for (int n = 0; n < tl->num_sources; n++) {
@@ -157,9 +140,12 @@ static struct demuxer *open_source(struct timeline *tl, char *filename)
         if (strcmp(d->stream->url, filename) == 0)
             return d;
     }
-    struct demuxer *d = open_file(filename, tl);
-    if (d)
+    struct demuxer *d = demux_open_url(filename, NULL, tl->cancel, tl->global);
+    if (d) {
         MP_TARRAY_APPEND(tl, tl->sources, tl->num_sources, d);
+    } else {
+        MP_ERR(tl, "EDL: Could not open source file '%s'.\n", filename);
+    }
     return d;
 }
 
@@ -311,6 +297,7 @@ static int try_open_file(struct demuxer *demuxer, enum demux_check check)
 {
     struct priv *p = talloc_zero(demuxer, struct priv);
     demuxer->priv = p;
+    demuxer->fully_read = true;
 
     struct stream *s = demuxer->stream;
     if (s->uncached_type == STREAMTYPE_EDL) {
