@@ -4,21 +4,20 @@
  * Copyright (c) 2001 laaz
  * Some code cleanup & realloc() by A'rpi/ESP-team
  *
- * This file is part of MPlayer.
+ * This file is part of mpv.
  *
- * MPlayer is free software; you can redistribute it and/or modify
+ * mpv is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
- * MPlayer is distributed in the hope that it will be useful,
+ * mpv is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License along
- * with MPlayer; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+ * with mpv.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <stdio.h>
@@ -547,8 +546,8 @@ static bool subreader_autodetect(stream_t *fd, struct MPOpts *opts,
             { sub_read_line_subrip, NULL, "subviewer" },
             { sub_read_line_subviewer, NULL, "subrip", "subrip" },
             { sub_read_line_ssa, NULL, "ssa", "ass-text" },
-            { sub_read_line_subviewer2, NULL, "subviewer 2.0" },
-            { sub_read_line_subrip09, NULL, "subrip 0.9" },
+            { sub_read_line_subviewer2, NULL, "subviewer2" },
+            { sub_read_line_subrip09, NULL, "subrip9" },
             { sub_read_line_mpl2, NULL, "mpl2" }
     };
     const struct subreader *srp;
@@ -791,6 +790,7 @@ static int d_open_file(struct demuxer *demuxer, enum demux_check check)
     subdata_free(sd);
 
     demuxer->seekable = true;
+    demuxer->fully_read = true;
 
     return 0;
 }
