@@ -1,19 +1,18 @@
 /*
- * This file is part of MPlayer.
+ * This file is part of mpv.
  *
- * MPlayer is free software; you can redistribute it and/or modify
+ * mpv is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
- * MPlayer is distributed in the hope that it will be useful,
+ * mpv is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License along
- * with MPlayer; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+ * with mpv.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef MP_COMMAND_LIST_H
@@ -49,6 +48,7 @@ enum mp_command_type {
     MP_CMD_OSD,
     MP_CMD_SCREENSHOT,
     MP_CMD_SCREENSHOT_TO_FILE,
+    MP_CMD_SCREENSHOT_RAW,
     MP_CMD_LOADFILE,
     MP_CMD_LOADLIST,
     MP_CMD_PLAYLIST_CLEAR,
@@ -73,6 +73,9 @@ enum mp_command_type {
     MP_CMD_MULTIPLY,
     MP_CMD_CYCLE_VALUES,
     MP_CMD_STOP,
+    MP_CMD_AUDIO_ADD,
+    MP_CMD_AUDIO_REMOVE,
+    MP_CMD_AUDIO_RELOAD,
 
     MP_CMD_ENABLE_INPUT_SECTION,
     MP_CMD_DISABLE_INPUT_SECTION,
@@ -83,6 +86,8 @@ enum mp_command_type {
 
     MP_CMD_DROP_BUFFERS,
 
+    MP_CMD_MOUSE,
+
     /// Audio Filter commands
     MP_CMD_AF,
     MP_CMD_AO_RELOAD,
@@ -106,6 +111,8 @@ enum mp_command_type {
     MP_CMD_HOOK_ADD,
     MP_CMD_HOOK_ACK,
 
+    MP_CMD_RESCAN_EXTERNAL_FILES,
+
     // Internal
     MP_CMD_COMMAND_LIST, // list of sub-commands in args[0].v.p
 };
