@@ -1,19 +1,18 @@
 /*
- * This file is part of MPlayer.
+ * This file is part of mpv.
  *
- * MPlayer is free software; you can redistribute it and/or modify
+ * mpv is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
- * MPlayer is distributed in the hope that it will be useful,
+ * mpv is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License along
- * with MPlayer; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+ * with mpv.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <stddef.h>
@@ -127,11 +126,26 @@ void uninit_sub_renderer(struct MPContext *mpctx)
 static void init_sub_renderer(struct MPContext *mpctx) {}
 void uninit_sub_renderer(struct MPContext *mpctx) {}
 
-void mp_ass_configure_fonts(struct ass_renderer *a, struct MPOpts *b,
-                            struct mpv_global *c, struct mp_log *d) {}
+static void mp_ass_configure_fonts(struct ass_renderer *a, struct osd_style_opts *b,
+                                   struct mpv_global *c, struct mp_log *d) {}
 
 #endif
 
+static void reset_subtitles(struct MPContext *mpctx, int order)
+{
+    int obj = order ? OSDTYPE_SUB2 : OSDTYPE_SUB;
+    if (mpctx->d_sub[order])
+        sub_reset(mpctx->d_sub[order]);
+    set_osd_subtitle(mpctx, NULL);
+    osd_set_text(mpctx->osd, obj, NULL);
+}
+
+void reset_subtitle_state(struct MPContext *mpctx)
+{
+    reset_subtitles(mpctx, 0);
+    reset_subtitles(mpctx, 1);
+}
+
 void uninit_stream_sub_decoders(struct demuxer *demuxer)
 {
     for (int i = 0; i < demuxer->num_streams; i++) {
@@ -177,21 +191,6 @@ static bool is_interleaved(struct MPContext *mpctx, struct track *track)
     return track->demuxer == mpctx->demuxer;
 }
 
-void reset_subtitles(struct MPContext *mpctx, int order)
-{
-    int obj = order ? OSDTYPE_SUB2 : OSDTYPE_SUB;
-    if (mpctx->d_sub[order])
-        sub_reset(mpctx->d_sub[order]);
-    set_osd_subtitle(mpctx, NULL);
-    osd_set_text(mpctx->osd, obj, NULL);
-}
-
-void reset_subtitle_state(struct MPContext *mpctx)
-{
-    reset_subtitles(mpctx, 0);
-    reset_subtitles(mpctx, 1);
-}
-
 void update_osd_sub_state(struct MPContext *mpctx, int order,
                           struct osd_sub_state *out_state)
 {
@@ -321,6 +320,8 @@ static void reinit_subdec(struct MPContext *mpctx, struct track *track,
     if (!track->preloaded && track->is_external && !opts->sub_clear_on_seek) {
         demux_seek(track->demuxer, 0, SEEK_ABSOLUTE);
         track->preloaded = sub_read_all_packets(dec_sub, track->stream);
+        if (track->preloaded)
+            demux_stop_thread(track->demuxer);
     }
 }
 
