@@ -17,6 +17,8 @@
 #include <stdio.h>
 #include <stdint.h>
 
+#include "osdep/io.h"
+
 #include "common/common.h"
 #include "common/msg.h"
 
@@ -125,7 +127,13 @@ int stream_file_cache_init(stream_t *cache, stream_t *stream,
     if (!opts->file || !opts->file[0] || opts->file_max < 1)
         return 0;
 
-    FILE *file = fopen(opts->file, "wb+");
+    if (!stream->seekable) {
+        MP_ERR(cache, "can't cache unseekable stream\n");
+        return -1;
+    }
+
+    bool use_anon_file = strcmp(opts->file, "TMP") == 0;
+    FILE *file = use_anon_file ? tmpfile() : fopen(opts->file, "wb+");
     if (!file) {
         MP_ERR(cache, "can't open cache file '%s'\n", opts->file);
         return -1;
