@@ -1,19 +1,18 @@
 /*
- * This file is part of MPlayer.
+ * This file is part of mpv.
  *
- * MPlayer is free software; you can redistribute it and/or modify
+ * mpv is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
- * MPlayer is distributed in the hope that it will be useful,
+ * mpv is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License along
- * with MPlayer; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+ * with mpv.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef MPLAYER_MP_IMAGE_H
@@ -31,11 +30,8 @@
 
 #define MP_PALETTE_SIZE (256 * 4)
 
-#define MP_IMGFIELD_ORDERED 0x01
 #define MP_IMGFIELD_TOP_FIRST 0x02
 #define MP_IMGFIELD_REPEAT_FIRST 0x04
-#define MP_IMGFIELD_TOP 0x08
-#define MP_IMGFIELD_BOTTOM 0x10
 #define MP_IMGFIELD_INTERLACED 0x20
 
 // Describes image parameters that usually stay constant.
@@ -48,6 +44,7 @@ struct mp_image_params {
     enum mp_csp colorspace;
     enum mp_csp_levels colorlevels;
     enum mp_csp_prim primaries;
+    enum mp_csp_trc gamma;
     enum mp_chroma_location chroma_location;
     // The image should be converted to these levels. Unlike colorlevels, it
     // does not describe the current state of the image. (Somewhat similar to
@@ -76,7 +73,7 @@ struct mp_image_params {
  *   image data. mp_image_make_writeable() will do that copy if required.
  */
 typedef struct mp_image {
-    unsigned int flags; // same as fmt.flags
+    int w, h;  // visible dimensions (redundant with params.w/h)
 
     struct mp_image_params params;
 
@@ -84,24 +81,12 @@ typedef struct mp_image {
     struct mp_imgfmt_desc fmt;
     enum mp_imgfmt imgfmt;
     int num_planes;
-    int chroma_x_shift; // horizontal
-    int chroma_y_shift; // vertical
 
-    int w,h;  // visible dimensions
     uint8_t *planes[MP_MAX_PLANES];
     int stride[MP_MAX_PLANES];
 
-    char * qscale;
-    int qstride;
     int pict_type; // 0->unknown, 1->I, 2->P, 3->B
     int fields;
-    int qscale_type; // 0->mpeg1/4/h263, 1->mpeg2
-
-    /* redundant */
-    int chroma_width;
-    int chroma_height;
-    int plane_w[MP_MAX_PLANES];
-    int plane_h[MP_MAX_PLANES];
 
     /* only inside filter chain */
     double pts;
@@ -111,6 +96,8 @@ typedef struct mp_image {
     void* priv;
 } mp_image_t;
 
+int mp_chroma_div_up(int size, int shift);
+
 struct mp_image *mp_image_alloc(int fmt, int w, int h);
 void mp_image_copy(struct mp_image *dmpi, struct mp_image *mpi);
 void mp_image_copy_attributes(struct mp_image *dmpi, struct mp_image *mpi);
@@ -127,6 +114,8 @@ void mp_image_crop_rc(struct mp_image *img, struct mp_rect rc);
 void mp_image_vflip(struct mp_image *img);
 
 void mp_image_set_size(struct mp_image *mpi, int w, int h);
+int mp_image_plane_w(struct mp_image *mpi, int plane);
+int mp_image_plane_h(struct mp_image *mpi, int plane);
 
 void mp_image_setfmt(mp_image_t* mpi, int out_fmt);
 void mp_image_steal_data(struct mp_image *dst, struct mp_image *src);
@@ -134,14 +123,12 @@ void mp_image_steal_data(struct mp_image *dst, struct mp_image *src);
 struct mp_image *mp_image_new_custom_ref(struct mp_image *img, void *arg,
                                          void (*free)(void *arg));
 
-struct mp_image *mp_image_new_external_ref(struct mp_image *img, void *arg,
-                                           void (*ref)(void *arg),
-                                           void (*unref)(void *arg),
-                                           bool (*is_unique)(void *arg),
-                                           void (*free)(void *arg));
-
 void mp_image_params_guess_csp(struct mp_image_params *params);
 
+char *mp_image_params_to_str_buf(char *b, size_t bs,
+                                 const struct mp_image_params *p);
+#define mp_image_params_to_str(p) mp_image_params_to_str_buf((char[80]){0}, 80, p)
+
 bool mp_image_params_valid(const struct mp_image_params *p);
 bool mp_image_params_equal(const struct mp_image_params *p1,
                            const struct mp_image_params *p2);
@@ -160,4 +147,9 @@ void mp_image_copy_fields_to_av_frame(struct AVFrame *dst,
 struct mp_image *mp_image_from_av_frame(struct AVFrame *av_frame);
 struct AVFrame *mp_image_to_av_frame_and_unref(struct mp_image *img);
 
+void memcpy_pic(void *dst, const void *src, int bytesPerLine, int height,
+                int dstStride, int srcStride);
+void memset_pic(void *dst, int fill, int bytesPerLine, int height, int stride);
+void memset16_pic(void *dst, int fill, int unitsPerLine, int height, int stride);
+
 #endif /* MPLAYER_MP_IMAGE_H */
