@@ -27,9 +27,10 @@ struct vo_cocoa_state;
 int vo_cocoa_init(struct vo *vo);
 void vo_cocoa_uninit(struct vo *vo);
 
-int vo_cocoa_config_window(struct vo *vo, uint32_t flags, void *gl_ctx);
+int vo_cocoa_config_window(struct vo *vo, uint32_t flags);
 
 void vo_cocoa_set_current_context(struct vo *vo, bool current);
+bool vo_cocoa_start_frame(struct vo *vo);
 void vo_cocoa_swap_buffers(struct vo *vo);
 int vo_cocoa_check_events(struct vo *vo);
 int vo_cocoa_control(struct vo *vo, int *events, int request, void *arg);
