@@ -35,10 +35,8 @@
 
 #include "config.h"
 
-#if HAVE_COCOA_APPLICATION
-# include "osdep/macosx_application.h"
-# include "osdep/macosx_application_objc.h"
-#endif
+#include "osdep/macosx_application.h"
+#include "osdep/macosx_application_objc.h"
 
 #include "options/options.h"
 #include "video/out/vo.h"
@@ -109,6 +107,14 @@ static void with_cocoa_lock_on_main_thread(struct vo *vo, void(^block)(void))
     });
 }
 
+static void with_cocoa_lock_on_main_thread_sync(struct vo *vo, void(^block)(void))
+{
+    struct vo_cocoa_state *s = vo->cocoa;
+    dispatch_sync(dispatch_get_main_queue(), ^{
+        with_cocoa_lock(s, block);
+    });
+}
+
 static void queue_new_video_size(struct vo *vo, int w, int h)
 {
     struct vo_cocoa_state *s = vo->cocoa;
@@ -279,7 +285,7 @@ void vo_cocoa_uninit(struct vo *vo)
 {
     struct vo_cocoa_state *s = vo->cocoa;
 
-    with_cocoa_lock_on_main_thread(vo, ^{
+    with_cocoa_lock_on_main_thread_sync(vo, ^{
         enable_power_management(s);
         cocoa_uninit_light_sensor(s);
         cocoa_rm_fs_screen_profile_observer(s);
@@ -462,13 +468,11 @@ static void create_ui(struct vo *vo, struct mp_rect *win, int geo_flags)
     [view signalMousePosition];
     s->adapter = adapter;
 
-#if HAVE_COCOA_APPLICATION
     cocoa_register_menu_item_action(MPM_H_SIZE,   @selector(halfSize));
     cocoa_register_menu_item_action(MPM_N_SIZE,   @selector(normalSize));
     cocoa_register_menu_item_action(MPM_D_SIZE,   @selector(doubleSize));
     cocoa_register_menu_item_action(MPM_MINIMIZE, @selector(performMiniaturize:));
     cocoa_register_menu_item_action(MPM_ZOOM,     @selector(performZoom:));
-#endif
 
     s->video = [[MpvVideoView alloc] initWithFrame:[s->view bounds]];
     [s->video setWantsBestResolutionOpenGLSurface:YES];
@@ -546,7 +550,7 @@ void vo_cocoa_release_nsgl_ctx(struct vo *vo)
     s->gl_ctx = nil;
 }
 
-int vo_cocoa_config_window(struct vo *vo, uint32_t flags, void *gl_ctx)
+int vo_cocoa_config_window(struct vo *vo, uint32_t flags)
 {
     struct vo_cocoa_state *s = vo->cocoa;
     with_cocoa_lock_on_main_thread(vo, ^{
@@ -632,13 +636,21 @@ static void draw_changes_after_next_frame(struct vo *vo)
     }
 }
 
+bool vo_cocoa_start_frame(struct vo *vo)
+{
+    struct vo_cocoa_state *s = vo->cocoa;
+
+    s->skip_swap_buffer = false;
+    return true;
+}
+
 void vo_cocoa_swap_buffers(struct vo *vo)
 {
     struct vo_cocoa_state *s = vo->cocoa;
 
     if (s->skip_swap_buffer && !s->waiting_frame) {
         s->skip_swap_buffer = false;
-        return;
+        s->pending_events |= VO_EVENT_EXPOSE;
     } else {
         [s->gl_ctx flushBuffer];
     }
@@ -771,11 +783,13 @@ int vo_cocoa_control(struct vo *vo, int *events, int request, void *arg)
             *(double *)arg = vo->cocoa->screen_fps;
             return VO_TRUE;
         }
+        break;
     case VOCTRL_GET_AMBIENT_LUX:
         if (vo->cocoa->light_sensor != IO_OBJECT_NULL) {
             *(int *)arg = vo->cocoa->last_lux;
             return VO_TRUE;
         }
+        break;
     }
     return VO_NOTIMPL;
 }
