@@ -471,8 +471,10 @@ static void wait_vo(struct vo *vo, int64_t until_time)
         pthread_mutex_unlock(&in->lock);
     } else {
         pthread_mutex_lock(&in->lock);
-        if (!in->need_wakeup)
-            mpthread_cond_timedwait(&in->wakeup, &in->lock, until_time);
+        if (!in->need_wakeup) {
+            struct timespec ts = mp_time_us_to_timespec(until_time);
+            pthread_cond_timedwait(&in->wakeup, &in->lock, &ts);
+        }
         in->need_wakeup = false;
         pthread_mutex_unlock(&in->lock);
     }
@@ -586,7 +588,7 @@ static bool render_frame(struct vo *vo)
     int64_t duration = in->frame_duration;
     struct mp_image *img = in->frame_queued;
 
-    if (!img && (!in->vsync_timed || in->paused || pts <= 0))
+    if (!img && (!in->vsync_timed || in->paused))
         goto nothing_done;
 
     if (in->vsync_timed && !in->hasframe)
@@ -608,14 +610,22 @@ static bool render_frame(struct vo *vo)
     if (!in->hasframe_rendered)
         duration = -1; // disable framedrop
 
-    // if the clip and display have similar/identical fps, it's possible that
+    // If the clip and display have similar/identical fps, it's possible that
     // we'll be very slightly late frequently due to timing jitter, or if the
     // clip/container timestamps are not very accurate.
-    // so if we dropped the previous frame, keep dropping until we're aligned
+    // So if we dropped the previous frame, keep dropping until we're aligned
     // perfectly, else, allow some slack (1 vsync) to let it settle into a rhythm.
+    // On low clip fps, we don't drop anyway and the slack logic doesn't matter.
+    // If the clip fps is more than ~5% above screen fps, we remove this slack
+    // and use "normal" logic to allow more regular drops of 1 frame at a time.
+    bool use_slack = duration > (0.95 * in->vsync_interval);
     in->dropped_frame = duration >= 0 &&
+                        use_slack ?
                             ((in->dropped_frame && end_time < next_vsync) ||
-                            (end_time < prev_vsync)); // hard threshold - 1 vsync late
+                            (end_time < prev_vsync)) // hard threshold - 1 vsync late
+                            :
+                            end_time < next_vsync; // normal frequent drops
+
 
     in->dropped_frame &= !(vo->driver->caps & VO_CAP_FRAMEDROP);
     in->dropped_frame &= (vo->global->opts->frame_dropping & 1);
@@ -651,7 +661,7 @@ static bool render_frame(struct vo *vo)
 
         MP_STATS(vo, "start video");
 
-        if (in->vsync_timed) {
+        if (vo->driver->draw_image_timed) {
             struct frame_timing t = (struct frame_timing) {
                 .pts        = pts,
                 .next_vsync = next_vsync,
@@ -961,6 +971,8 @@ void vo_event(struct vo *vo, int event)
     pthread_mutex_lock(&in->lock);
     if ((in->queued_events & event & VO_EVENTS_USER) != (event & VO_EVENTS_USER))
         mp_input_wakeup(vo->input_ctx);
+    if (event)
+        wakeup_locked(vo);
     in->queued_events |= event;
     in->internal_events |= event;
     pthread_mutex_unlock(&in->lock);
