@@ -483,13 +483,11 @@ static void *screensaver_thread(void *arg)
         // don't queue multiple wakeups
         while (!sem_trywait(&x11->screensaver_sem)) {}
 
-        if (mp_cancel_test(x11->screensaver_terminate))
+        if (atomic_load(&x11->screensaver_terminate))
             break;
 
         char *args[] = {"xdg-screensaver", "reset", NULL};
-        if (mp_subprocess(args, x11->screensaver_terminate, NULL, NULL,
-                          NULL, &(char*){0}))
-        {
+        if (mp_subprocess(args, NULL, NULL, NULL, NULL, &(char*){0})) {
             MP_WARN(x11, "Disabling screensaver failed.\n");
             break;
         }
@@ -515,13 +513,12 @@ int vo_x11_init(struct vo *vo)
     };
     vo->x11 = x11;
 
-    x11->screensaver_terminate = mp_cancel_new(x11);
     sem_init(&x11->screensaver_sem, 0, 0);
     if (pthread_create(&x11->screensaver_thread, NULL, screensaver_thread, x11)) {
-        x11->screensaver_terminate = NULL;
         sem_destroy(&x11->screensaver_sem);
         goto error;
     }
+    x11->screensaver_thread_running = true;
 
     x11_error_output = x11->log;
     XSetErrorHandler(x11_errorhandler);
@@ -731,8 +728,8 @@ void vo_x11_uninit(struct vo *vo)
         XCloseDisplay(x11->display);
     }
 
-    if (x11->screensaver_terminate) {
-        mp_cancel_trigger(x11->screensaver_terminate);
+    if (x11->screensaver_thread_running) {
+        atomic_store(&x11->screensaver_terminate, true);
         sem_post(&x11->screensaver_sem);
         pthread_join(x11->screensaver_thread, NULL);
         sem_destroy(&x11->screensaver_sem);
@@ -1389,6 +1386,9 @@ static void vo_x11_map_window(struct vo *vo, struct mp_rect rc)
     vo_x11_selectinput_witherr(vo, x11->display, x11->window, events);
     XMapWindow(x11->display, x11->window);
 
+    if (vo->opts->fullscreen && (x11->wm_type & vo_wm_FULLSCREEN))
+        x11_set_ewmh_state(x11, "_NET_WM_STATE_FULLSCREEN", 1);
+
     vo_x11_xembed_update(x11, XEMBED_MAPPED);
 }
 
@@ -1752,10 +1752,12 @@ int vo_x11_control(struct vo *vo, int *events, int request, void *arg)
     case VOCTRL_GET_ICC_PROFILE: {
         if (!x11->pseudo_mapped)
             return VO_NOTAVAIL;
+        int cx = x11->winrc.x0 + (x11->winrc.x1 - x11->winrc.x0)/2,
+            cy = x11->winrc.y0 + (x11->winrc.y1 - x11->winrc.y0)/2;
         int screen = 0; // xinerama screen number
         for (int n = 0; n < x11->num_displays; n++) {
             struct xrandr_display *disp = &x11->displays[n];
-            if (disp->overlaps) {
+            if (mp_rect_contains(&disp->rc, cx, cy)) {
                 screen = n;
                 break;
             }
